var _ = require('underscore');
var deferred = require('deferred');

module.exports = function(self, app, checkAdmin) {
var defaultRTMPPluginValues = {
    "user_agent":"FMLE"
    ,"copyright":"Teradek"
    ,"device":"sputnik"
    ,"compat":"0"
    ,"username":""
    ,"password":""
    ,"auto_start":"1"
};


    app.post('/loadRTMPPlugin', checkAdmin, function(req, res){
       var sid = req.body.sid;
       try{
       var params = req.body;
           _.defaults(params, defaultRTMPPluginValues);
           params =_.map(params,function(v,k){ return k+'='+v });
       }
       catch(e){
           console.log(e)
           res.json({error:'unable to parse rtmp params'});
           return false;
       }
       var sess = self.sess(sid);
       sess.iface('E').enqueueOne('dev:load-streamToCdn-plugin', function (session) {
            self.plugins.streamToCdn(session, sid, params)(function (pluginId) {
                console.log('loadStreamToCdnPlugin() loaded streamToCdn plugin', pluginId);
                res.json({success:pluginId});
            },function (err) {
                console.log('loadStreamToCdnPlugin() could not load streamToCdn plugin', err.toString());
                res.json({error:err.toString()});
            }).done();
            session = null;
        });
    });


    app.post('/:sid/loadRTSPPlugin', checkAdmin, function(req, res){
        var sid = req.params.sid;
        var sess = self.sess(sid);
        var opts = req.body;

       sess.iface('E').enqueueOne('dev:load-rtsp-server', function (session) {
            self.plugins.loadRTSPServer(session, sid, opts)(function (pluginId) {
                    res.json({success:pluginId});
                }, function (err) {
                        console.log('ERROR: could not load RTSP Server plugin', err.toString());
                        res.json({error:err.toString()});

                    });
            },function (err) {
                console.log('ERROR: could not load RTSP Server plugin', err.toString());
                res.json({error:err.toString()});
            });
    });


    app.get('/loadSnapshotPlugin', checkAdmin, function(req, res){
       var sid = req.query.sid;
       var sess = self.sess(sid);

       sess.iface('E').enqueueOne('dev:load-snapshot-plugin', function (session) {
            self.plugins.loadSnap(session)(function (pluginId) {
                    res.json({success:pluginId});
                }, function (err) {
                        console.log('ERROR: could not load Snapshot plugin', err.toString());
                        res.json({error:err.toString()});

                    });
            },function (err) {
                console.log('ERROR: could not load Snapshot plugin', err.toString());
                res.json({error:err.toString()});
            });
    });



    app.get('/connectRTMP', checkAdmin, function(req, res){
        var sid = req.query.sid;
        var sess = self.sess(sid);
        sess.iface('E').enqueueOne('dev:connect-rtmp', function (session) {
            //rtmpwriter.disconnect();
        });
    });


    /* doesn't appear to work right now */
    app.get('/disconnectRTMP', checkAdmin, function(req, res){
        var sid = req.query.sid;
        var plugin_id = req.query.plugin_id;
        var stream_id = req.query.stream_id;
        var sess = self.sess(sid);

        sess.iface('E').enqueueOne('dev:disconnect-rtmp', function (session) {
            session.rtmpwriter(stream_id).disconnect(plugin_id);
            //rtmpwriter.disconnect();
        });
    });

    app.get('/getRTMPState', checkAdmin, function(req, res){
        var sid = req.query.sid;
        var plugin_id = req.query.plugin_id;
        var stream_id = req.query.stream_id;
        var sess = self.sess(sid);

        sess.iface('E').enqueueOne('dev:getState-rtmp', function (session) {
            console.log('GET STATE', plugin_id, sid)
            session.rtmpwriter(stream_id).getState(plugin_id)(function (state) {
                console.log('getplugin state', state);
                res.json({success:state});
            },function (err) {
                console.log('failed to get plugin state', err.toString());
                res.json({error:err.toString()});
            });
        });
    });

    app.get('/unloadPlugin', checkAdmin, function(req, res){
        var sid = req.query.sid;
        var plugin_id = req.query.plugin_id;
        var sess = self.sess(sid);

        sess.iface('E').enqueueOne('dev:unload-plugin', function (session) {
            session.unloadPlugin(plugin_id)(function (pluginId) {
                console.log('unloadingPlugin() UNloaded plugin', pluginId);
                res.json({success:pluginId});
            },function (err) {
                console.log('FAILED: unloadingPlugin() could not unload plugin',pluginId, err.toString());
                res.json({error:err.toString()});
            });
            //rtmpwriter.disconnect();
        });
    });


    //lists only plugins that were loaded by this instance of the webserver
    //if the webserver is restarted or another process loads plugins
    //then this instance will know nothing about that plugin
    app.get('/:sid/pluginParams', checkAdmin, function(req, res){
        var sid = req.params.sid;
        var sess = self.sess(sid);
        sess.iface('E').enqueueOne('dev:list-plugins', function (session) {
            res.json(session.listPlugins());

       }, function(err){
            console.log('listPluginParams ERROR');
            res.json({error:err.toString()});
       });
    });

    app.get('/:sid/plugins', checkAdmin, function(req, res){
        var sid = req.params.sid;
        var sess = self.sess(sid);
        sess.iface('E').enqueueOne('dev:list-plugins', function (session) {
            if( 'unfiltered' in req.query ){
                session.listPluginParams()(function(data){res.json(data);});
            }
            else{
                session.listRunningPluginParams()(function(data){res.json(data);});
            }
        },function(err){
                console.log('listPluginParams ERROR');
                res.json({error:err.toString()});
        });
    });


    self.loadCompatiblePlugin = function( session, sid ){
        var def = deferred();
        var plugins = session.listPlugins();

        if( 'rtp' in plugins && sid in plugins.rtp ){
            def.resolve();
        }
        else{
            var sess = self.sess(sid);
            var opts = {};
            self.plugins.loadRTSPServer(session, sid, opts)(function (pluginId) {
                    def.resolve({success:pluginId});
                }, function (err) {
                        console.log('ERROR: could not load RTSP Server plugin', err.toString());
                        def.reject({error:err.toString()});

                    });
        }
        return def.promise;
    };

    //pair sid with what? specifiy which plugin to pair
    app.get('/:sid/pair', checkAdmin, function(req, res){
        var sid = req.params.sid;
        var sess = self.sess(sid);
        var pairWith = self.sess(req.query['with']);
        sess.iface('E').enqueueOne('dev:list-plugins', function( session ){
            self.loadCompatiblePlugin( session, sid )(function(){
                var plugins = session.listPlugins();
                var rtp_settings = {};

                var plugin_settings = plugins.rtp[sid];
                //convert rtp argument string to json obj
                _.each(plugin_settings[0].data, function(v, k){
                    var matches = v.match(/^([^=]+)=(.+)$/);
                    rtp_settings[matches[1]] = matches[2];
                });

                //get sessid && port of rtp server
                //pairing forces TCP proto for RTSP
                //username & password?
                //udp or tcp?
                //host needs to be sputnik itself via public ip
               var settings = {
                    "VideoPlayer.Settings.1.RTSP.proto": 'tcp',
                    "VideoPlayer.Settings.1.mode":'RTSP'
                   ,"VideoPlayer.Settings.1.RTSP.host":"TBD"
                   ,"VideoPlayer.Settings.1.RTSP.port":rtp_settings.prt
                   ,"VideoPlayer.Settings.1.RTSP.name":rtp_settings.ses
               };

               pairWith.iface('C').enqueueOne('dev:emit-setSettings', function (session) {
               session.getSettings({q:"Bond.Configuration.Server.host"}).then(function(host){
                   var host_val_matches = JSON.stringify(host).match(/{"Bond":{"Configuration":{"Server":{"host":"(.+)"}}}}/);
                   var host_val = host_val_matches[1];
                   settings["VideoPlayer.Settings.1.RTSP.host"] = host_val;
                   session.setSettings(settings).then(function(data){
                        res.json(data);
                    }, function(data){
                         res.json({error:'Failed to set settings', data:arguments});
                        });
                   });
               });
            });


            //if running plugin has decoder supported stream type use it
            //otherwise create a new supported plugin (RTP or MPEG-TS)
            //if(){}
            });

    });

    app.get('/:sid/runningPlugins', checkAdmin, function(req, res){
        var sid = req.params.sid;
        var sess = self.sess(sid);
        sess.iface('E').enqueueOne('dev:list-plugins', function (session) {
            session.listRunningPluginIds()(function(data){
                res.json(data);
            });
        },function(err){
                console.log('listRunningPluginIds ERROR');
                res.json({error:err.toString()});
        });
    });

    app.get('/:sid/mainPlugins', checkAdmin, function(req, res){
        var sid = req.params.sid;
        var sess = self.sess(sid);
        sess.iface('E').enqueueOne('dev:list-plugins', function (session) {
            session.listMainPluginParams()(function(data){
                res.json(data);
            });
        },function(err){
                console.log('mainPlugins ERROR');
                res.json({error:err.toString()});
        });
    });

};