var properties = require('properties')
            ,_ = require('underscore')
           ,fs = require('fs')
    ,validator = require('validator')
    ,path      = require('path')
     ,deferred = require('deferred');

/*_.extend(validation.validators, {
  portnumber: function(value, attr, customValue, model) {
    return validaton.validators.range(value, attr, [1,50009], model);
  }
  ,string_set: function(value, attr, customValue, model) {
    return this.oneOf(value, attr, customValue, model);
  }
});*/

validator.extend( 'is_integer_range', function( input, min, max ){
    var input = validator.toInt(input);
    return input >= min && input <= max;
});

validator.extend( 'is_valid_friendly_name', function( input ){
    return validator.matches(input, /^[A-z0-9 ]+$/) && validator.isLength(input, 1, 32);
});

validator.extend( 'is_valid_company_name', function( input ){
    return validator.matches(input, /^[A-z0-9 ]+$/) && validator.isLength(input, 1, 255);
});

validator.extend( 'is_email', function( input ){
    return validator.matches(input, /^(([^<>()\[\]\\.,;:\s@"]+(\.[^<>()\[\]\\.,;:\s@"]+)*)|(".+"))@((\[[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}\.[0-9]{1,3}])|(([a-zA-Z\-0-9]+\.)+[a-zA-Z]{2,}))$/);
});



validator.extend( 'is_portnumber', function( input ){
    return validator.is_integer_range(input, 1, 50009);
});

validator.extend( 'is_string_set', function( input, values ){
    return validator.isIn(input,values);
});

validator.extend( 'is_alphanumeric', function( input ){
    return validator.isAlphanumeric(input);
});

validator.extend( 'is_blank', function( input ){
    return true;
});

validator.extend( 'is_base64', function( input ){
    return validator.matches(input, /^([A-Za-z0-9+/]{4})*([A-Za-z0-9+/]{4}|[A-Za-z0-9+/]{3}=|[A-Za-z0-9+/]{2}==)$/);
});

validator.extend( 'is_write_path', function( input ){
    var p = path.normalize( input );
    if( !(/[^\0]+/.test(p)) ){
        return false;
    }
    var pathParts =  p.split(path.sep);
    //make sure each path is an alphanumeric character
    return _.every(pathParts, function( part ){
        if( part == ''){return true;}
        return validator.isAlphanumeric( part );
    });
});



var CONFIG_FILE = "/etc/sputnik.conf";
var configMeta = {
    friendly_name:{
         label:"Friendly Name"
        ,description:"A name to help uniquely identify this Sputnik server instance."
        ,default:"teradek-sputnik"
        ,type:['valid_friendly_name']
    },
    input_port:{
         label:"Video input port"
        ,description:"The input port that Sputnik listens on for incoming connections from"
        ,default:5111
        ,type:['portnumber']
    }
    ,web_port:{
         label:"Webserver port"
        ,description:"The port that Sputnik dashboard is presented on"
        ,default:1957
        ,type:['portnumber']
    }
    ,redis_port:{
         label:"Database port"
        ,description:"The port that Sputnik Redis database listens on"
        ,default:6379
        ,type:['portnumber']
    }
    ,log_size:{
         label:"Server Log file size"
        ,description:"Maximum size of server log file before rotation"
        ,default:10485760
        ,type:['integer_range']
        ,max:1048576000
        ,min:500
    }
    ,log_number:{
         label:"Server Log file size"
        ,description:"Maximum number of server log files before deleting oldest"
        ,default:5
        ,type:['integer_range']
        ,max:500
        ,min:1
    }
    ,core_client:{
         label:"Core client enabled"
        ,description:"Enable Core uplink"
        ,default:'no'
        ,choices:['yes','no']
        ,type:['string_set']
    }
    ,rtp_server:{
         label:"Enable RTP Server"
        ,description:"# A local RTP Server for monitoring incoming video streams"
        ,default:"no"
        ,choices:['yes','no']
        ,type:['string_set']
    }
    ,rtp_server_port:{
         label:"RTP Server Port"
        ,description:"The port that the RTP Server listens on"
        ,default:554
        ,type:['portnumber']
    }
    ,rtp_server_username:{
         label:"RTP Server username"
        ,description:"The username used to authenticate access to the RTP Server streams. Leave empty for no authentication."
        ,default:''
        ,type:['blank','alphanumeric']
    }
    ,rtp_server_password:{
         label:"RTP Server password"
        ,description:"The password used to authenticate access to the RTP Server streams. Leave empty for no authentication."
        ,default:''
        ,type:['blank']
    }
    ,view_only:{
        label:"View Only"
        ,description:"will probably be deprecated"
        ,choices:['yes','no']
        ,type:['string_set']
    }
    ,upload_path:{
        label:"Upload Path"
        ,description:"path to upload files"
        ,type:['write_path']
    }
    ,core_company_name:{
        label:"Company Name"
        ,description:"Company Name associated with Core Account"
        ,type:['valid_company_name']
    }
    ,core_user_email:{
        label:"Core Username"
        ,description:"User Email used to connect with core"
        ,type:['is_email']
    }
}

module.exports = {
        settings:null
        ,load: function( options ){
          var _this = this;

          var defaults = {
              configFile: CONFIG_FILE
              ,def: deferred()
          };
          options = options ? options : {};
          var opts = _.defaults(options, defaults);

          var properties_options = {path:true, reviver:function( k, v, section ){
              if( typeof v == 'string' ){
                v = v.replace(/"/g, ""); //remove quotes
              }
              return v;
          }};
          properties.parse( opts.configFile, properties_options,
              function( error, settings ){
                if( error ){
                    console.log('Error reading config file:', opts.configFile, error);
                    return opts.def.resolve(error);
                }
                _this.settings = settings;
                opts.def.resolve(settings);
          });
          return opts.def.promise;
        }
        ,get: function(){
            var def = deferred();
            this.load( {def:def});
            return def.promise;
        }
        ,geti:function(){
            var def = deferred();
            var _this = this;
            this.get()(function(settings){
                var ret = _.map(configMeta, function(v,k){
                    var newObj = _.extend({id:k}, v);
                    if( settings[k] ){
                        _.extend(newObj, {value:settings[k]});
                    }
                    return newObj;
                });
                def.resolve(ret);
            })
            return def.promise;
        }
        ,validate:function( settings ){
            var error_status = {};
            var mergeSettings = _.map(settings, function(v, k){
                if( configMeta[k] ){
                    var obj = configMeta[k];
                    obj.value = v;
                    obj.key = k;
                    return obj;
                }
                else{
                    console.log('ERROR: no meta definition for ', k);
                    return { key: k, undefinedSetting:true };
                }
            });

            _.each( mergeSettings, function( v ){
                if( 'undefinedSetting' in v ){
                    if( !(error_status.undefinedSettings)){ error_status.undefinedSettings = []}
                    error_status.undefinedSettings.push({field:v.key});
                    return false;
                }
                _.every(v.type, function(type){
                    var args = [];
                    args.push(v.value);
                    if( 'choices' in v ){
                         args.push(v.choices);
                    }
                    if( 'min' in v ){
                        args.push(v.min);
                    }
                    if( 'max' in v ){
                        args.push(v.max);
                    }

                    if( validator.trim(v.value) == '' && type == 'blank' ){
                        return false;
                    }

                    if( validator['is_'+type] ){
                        if(!(validator['is_'+type].apply(null, args)) ){
                            error_status.error = true;
                            if( !(error_status.errorMsgs)){ error_status.errorMsgs = []}
                            error_status.errorMsgs.push({name:v.label, msg:"invalid value for "+type, field:v.key});
                            return false;
                        }
                    }
                    return true;
                })
            });
            return error_status;
        }
        ,set: function( settings ){
            //console.log("trying to set these sputnik settings", settings);
            var def = deferred();

            var errors = this.validate(settings);
            if( 'undefinedSettings' in errors ){
                _.each(errors.undefinedSettings, function(v){
                    delete settings[v.field];
                });
            }

            if( 'error' in errors ){
                console.log('config file reader found the following errors', errors);
                def.reject(new Error(JSON.stringify(errors)));
                return def.promise;
            }

            fs.readFile(CONFIG_FILE, 'utf8', function (err,data) {
              if (err) {
                console.log(err);
                def.resolve(err);
              }
              var result = data;
              _.each(settings, function(v, k){
                  var regex = new RegExp('^'+k+'\\s*=["]*.*["]*$', 'gm');

                  if( result.match(regex) ){
                    result = result.replace( regex, k+'="'+v+'"' );
                  }
                  else{
                      console.log( 'Adding the following setting to conf file since it does not current exist:', k, v );
                      result += k+'="'+v+'"'+"\n";
                  }
              });

              fs.truncate(CONFIG_FILE, 0, function(){
                  fs.writeFile(CONFIG_FILE, result, 'utf8', function (err) {
                     if (err){
                        console.log(err);
                        def.resolve(err);
                     }
                     def.resolve(settings);
                  });
              });
            });
            return def.promise;
        }
        ,needToMergeNewConfig: function(){
            var debMergeFile = '/etc/sputnik.conf.dpkg-dist';
            var debMergeOldFile = '/etc/sputnik.conf.dpkg-old';
            var rpmMergeFile = '/etc/sputnik.conf.rpmsave';
            if( fs.existsSync( debMergeFile ) ){
                return {file:debMergeFile, type:'new'};
            }
            else if( fs.existsSync( debMergeOldFile ) ){
                return {file:debMergeOldFile, type:'old'};
            }
            else if( fs.existsSync( rpmMergeFile ) ){
                return {file:rpmMergeFile, type:'old'};
            }
            return false;
        }
        ,mergeNewConfig: function(){

            var def = deferred();
            var _this = this;
            var needToMergeData = _this.needToMergeNewConfig();

            if( needToMergeData ){
                deferred( _this.load(), _this.load( {configFile: needToMergeData.file} ) )(
                    function( results ){
                        if( needToMergeData.type == 'new' ){
                            var oldConfig = results[0];
                            var newConfig = results[1];
                        }
                        else{
                            var oldConfig = results[1];
                            var newConfig = results[0];
                        }

                        try{
                            var mergedConfig = _.defaults( oldConfig, newConfig );
                            var invalidMergedSettings = _this.validate(mergedConfig);

                            //for each error in the merged validation default to the default new config value
                            if( 'error' in invalidMergedSettings ){
                                console.log('The following configuration(s) are no longer valid '+
                                             'and will be replaced by default values in the new configuration:',
                                             invalidMergedSettings.errorMsgs );

                                _.each(invalidMergedSettings.errorMsgs, function( v ){
                                    mergedConfig[v.field] = newConfig[v.field];
                                });
                            }
                            _this.set( mergedConfig )(function(){
                                fs.unlink( needToMergeData.file, function( err ){
                                    if( err ){
                                        def.resolve( err );
                                    }
                                    else{
                                        def.resolve( mergedConfig );
                                    }
                                });
                            });
                        }
                        catch(e){
                            console.log('Error merging config files', e);
                            def.resolve( new Error("unable to merge config files") );
                        }
                    }
                );
            }
            def.resolve(true);
            return def.promise;
        }
    }