/*
 * Copyright, 2012, Teradek
 * 
 * All rights reserved.
 * Contact Teradek for licensing.
 */

var cli = require('Node-CLI/node-cli.js'),
	colors = require('colors.js');


function sessions(config, server, session, startrow, startcol) {

	var row = startrow, col = startcol
	var sess = require('redis').createClient(config.port, config.host)

    sess.psubscribe("server:" + server + ":sess:" + session + ":stats")

	sess.on("psubscribe", function(pattern, count) {
		
        cli.move(row+0, col+0).write('Session'.bold.blue);
		cli.move(row+0, col+10).write('Connections'.bold.blue);
		cli.move(row+0, col+22).write('Bitrate'.bold.blue);
		cli.move(row+0, col+32).write('Queue Len'.bold.blue);
		cli.move(row+0, col+44).write('Skip Frames'.bold.blue);
	});

	sess.on("pmessage", function(pattern, channel, message) {
		var stats = JSON.parse(message);
        var now = new Date();
        var date = new Date(stats.ts*1000);

        cli.move(row+1, col).clearNext(80);
        cli.move(row+1, col+0).clearNext(12).write(session);
		cli.move(row+1, col+10).clearNext(12).write(stats.num_conn);
		cli.move(row+1, col+22).clearNext(12).write(stats.bitrate);
		cli.move(row+1, col+32).clearNext(12).write(stats.qlen);
		cli.move(row+1, col+44).clearNext(12).write(stats.skip_frames);
	});
}

function connections(config, server, session, startrow, startcol) {

	var row = startrow, col = startcol
	var conn = require('redis').createClient(config['port'], config['host'])
	
    conn.psubscribe("server:" + server + ":sess:" + session + ":conn:*:stats")

	conn.on("psubscribe", function(pattern, count) {
		cli.move(row, col).write("Conn".bold.blue)
		cli.move(row, col+6).write("IF".bold.blue)
		cli.move(row, col+12).write("IP".bold.blue)
		cli.move(row, col+28).write("State".bold.blue)
		cli.move(row, col+34).write("Bitrate".bold.blue)
		cli.move(row, col+44).write("RTD".bold.blue)
		cli.move(row, col+54).write("Late".bold.blue)
	});

	conn.on("pmessage", function(pattern, channel, message) {
        var conn_stats = JSON.parse(message);
        var conn_no = parseInt(channel.match(/server:.*:sess:.*:conn:([0-9]+):stats/)[1]);
        
        cli.move(row+1+conn_no, col+0).clearNext(80);
		cli.move(row+1+conn_no, col+1).write(conn_no);
		cli.move(row+1+conn_no, col+6).write(conn_stats.name);
		cli.move(row+1+conn_no, col+12).write(conn_stats.address);
		cli.move(row+1+conn_no, col+28).write(conn_stats.state);
		cli.move(row+1+conn_no, col+34).write(conn_stats.bitrate);
		cli.move(row+1+conn_no, col+44).write(conn_stats.rt_delay);
		cli.move(row+1+conn_no, col+54).write(conn_stats.late_frames);
	});
}

function display(config, server, session, row, col) {
	
	sessions(config, server, session, row, col)
	connections(config, server, session, row+3, col+10)
	
}

module.exports = {
	"display" : display
}
