var _ = require('underscore'),
    redis = require('redis'),
    deferred = require('deferred'),
    Events = require('events').EventEmitter,
    ioctl = require('./ioctl'),
    events = new Events,
    idTypes = ['cube', 'aux', 'bond', 'sputnik'],
    sessions = {},
    callbacks = {},
    ctlids = {},
    reqNum = 100,
    OPEN = 0,
    CLOSE = 1,
    sub,
    session;

events.setMaxListeners(0);

function isJSON(str) {
    try {
        JSON.parse(str);
    } catch (e) {
        return false;
    }

    return true;
}

function getCtlIds(sid, iface) {
    var def = deferred();

    if (ctlids[sid] && ctlids[sid][iface] != null) {
        (function () {
            if (ctlids[sid][iface] === false) return setTimeout(arguments.callee, 0);
            def.resolve(ctlids[sid][iface])
        })();
        return def.promise;
    }

    ctlids[sid] = ctlids[sid] || {};
    ctlids[sid][iface] = false;

    console.log('Getting CtlIds for ' + sid + ':' + iface);

    var ids = {};

    deferred.map(idTypes, function (idType) {
        reqNum++;

        var sess = 'sess:' + sid + ':' + sessions[sid][iface],
            _def = deferred(),
            _pub = redis.createClient(),
            _sub = redis.createClient(),
            reqStr = 'get_ioctl_id:' + reqNum + ':' + idType,
            resStr = 'reply:' + reqNum + ':';

        _sub.subscribe(sess);
        _sub.on('message', function (ch, data) {
            if (data.indexOf(resStr) === 0) {
                console.log('Got id for ' + idType);

                ids[idType] = data.split(':').pop();

                _sub.quit();
                _pub.quit();

                _def.resolve(1);

            }
        });

        setTimeout(function () {
            _pub.publish(sess, reqStr);
        }, 200);

        return _def.promise;
    })(function () {

        console.log('GotCtlIds for ' + sid + ':' + iface);

        ctlids[sid][iface] = ids;

        def.resolve(ids);
    }, function (err) {
        delete ctlids[sid][iface];
        def.resolve(err);
    });

    return def.promise;
}

function executeCallbacks(sid, iface) {
    var sessionExst = !!(sessions[sid] && sessions[sid][iface]),
        callbacksExst = !!(callbacks[sid] && callbacks[sid][iface]),
        ctlidsExst = !!(ctlids[sid] && ctlids[sid][iface] && Object.keys(ctlids[sid][iface]).length > 0),
        cb;
    if (!sessionExst) console.log('session:warning interface ' + iface + ' not found on session ' + sid);
    if (sessionExst && callbacksExst && ctlidsExst) {
        _.each(callbacks[sid][iface], function (callbacks, jobName) {
            while (cb = callbacks.shift()) {
                var opts = {
                        sid: sid,
                        iface: iface,
                        ctlids: ctlids[sid][iface],
                        ch: 'sess:' + sid + ':' + sessions[sid][iface]
                    },
                    sess = ioctl(opts);

                //console.log('executeCallbacks()', sid, iface, jobName);
                cb(sess);
            }
        });
    }
}

function updateSession(sid, ifaces, action) {
    var openSess = function (sid, iface) {

            if (!(ctlids[sid] && ctlids[sid][iface])) {
                getCtlIds(sid, iface)(function (ctlIds) {
                    executeCallbacks(sid, iface);
                }, function (err) {
                    console.log(err);
                });
            } else {
                executeCallbacks(sid, iface);
            }
        },
        closeSess = function (sid, iface) {
            delete sessions[sid][iface];
        },
        iface;

    sessions[sid] = sessions[sid] || {};
    callbacks[sid] = callbacks[sid] || {};

    switch (action) {
        case OPEN:

            for (var i = 0, j = ifaces.length; i < j; i++) {
                iface = ifaces.charAt(i).toUpperCase();
                sessions[sid][iface] = ifaces;
                openSess(sid, iface);
            }
            //if (ctlids[sid] && ctlids[sid][iface]){
                events.emit('open_session', sid, ifaces);
            //}
            break;

        case CLOSE:
            for (var i = 0, j = ifaces.length; i < j; i++) {
                iface = ifaces.charAt(i).toUpperCase();
                closeSess(sid, iface);
            }
            events.emit('close_session', sid, ifaces);
            break;
    }
}


/* --- Return API --- */

session = module.exports = function (sid) {
    sid = String(sid);
    callbacks[sid] = callbacks[sid] || {};
    //console.log('callbacks[sid]', sid)
    return {
        iface: function (iface) {
            iface = iface.toUpperCase();
            callbacks[sid][iface] = callbacks[sid][iface] || {};

            return {
                enqueue: function (jobName, cb) {
                    callbacks[sid][iface][jobName] = callbacks[sid][iface][jobName] || [];
                    callbacks[sid][iface][jobName].push(cb);
                    executeCallbacks(sid, iface);
                },

                enqueueOne: function (jobName, cb) {
                    callbacks[sid][iface][jobName] = [cb];
                    executeCallbacks(sid, iface);
                },

                flush: function (jobName) {
                    jobName && callbacks[sid][iface][jobName] ? delete callbacks[sid][iface][jobName] : delete callbacks[sid][iface];
                }
            }
        },

        flush: function () {
            delete callbacks[sid];
        }
    }
}

session.events = events;

session.init = function () {

    /* --- Subscribe to session events on Redis --- */

    sub = redis.createClient();

    sub.on('error', function () {
        console.log('utils/session.js: Error creating Redis client...');
    });

    sub.psubscribe('sess:*');

    sub.on('pmessage', function (p, ch, data) {

        var parts = ch.split(':'),
            sid = parts[1],
            ifaces = parts[2];

        // ignore sessions like "sess:1:S"
        if (ifaces === 'S' || ifaces === 'undefined') {
            return;
        }

        if (data === 'close_session')
            return updateSession(sid, ifaces, CLOSE);
        updateSession(sid, ifaces, OPEN);
        if (data === 'open_session') return;
        if (isJSON(data))
            return events.emit('sess:' + sid, ch, JSON.parse(data));
        if (/^reply:\d+:(\{.*?\})$/.test(data))
            return events.emit('sess:' + sid, ch, JSON.parse(/^reply:\d+:(\{.*?\})$/.exec(data)[1]));
    });
}